
#include <cmath>
#include <cstdio>


class Vector
{
 private:

 double X0;
 double Y0;
 double X1;
 double Y1;

 int ccw (double x0, double y0, double x1, double y1, double x2, double y2)
 {
  double dx1, dx2, dy1, dy2;
  
  dx1 = x1 - x0;
  dy1 = y1 - y0;
  dx2 = x2 - x0;
  dy2 = y2 - y0;
  if (dx1*dy2 > dy1*dx2)
    return +1;
  if (dx1*dy2 < dy1*dx2)
    return -1;
  if ((dx1*dx2 < 0) || (dy1*dy2 < 0))
    return -1;
  if ((dx1*dx1+dy1*dy1) < (dx2*dx2+dy2*dy2))
    return +1;
  return 0;
}
 
 public:
 
 Vector(void) {X0=0; Y0=0; X1=0; Y1=0;}
 Vector(double x0, double y0, double x1, double y1) {X0=x0; Y0=y0; X1=x1; Y1=y1;}
 
 void setX0(double x) {X0=x;}
 void setY0(double y) {Y0=y;}
 void setX1(double x) {X1=x;}
 void setY1(double y) {Y1=y;}
 double getX0(void) {return X0;}
 double getY0(void) {return Y0;}
 double getX1(void) {return X1;}
 double getY1(void) {return Y1;}

 bool equals(Vector v)
 {
  if (X0==v.getX0() && Y0==v.getY0() && X1==v.getX1() && Y1==v.getY1())
    return true;
  else
	  return false;
 }
 
 double length(void)
 {
  return sqrt(pow(X0-X1,2)+pow(Y0-Y1,2));
 }

 bool intersect(Vector v)
 {
  return ((ccw(X0,Y0,X1,Y1,v.getX0(),v.getY0())*ccw(X0,Y0,X1,Y1,v.getX1(),v.getY1())) <0)
         && ((ccw(v.getX0(),v.getY0(),v.getX1(),v.getY1(),X0,Y0)*ccw(v.getX0(),v.getY0(),v.getX1(),v.getY1(),X1,Y1)) <0);
 }
};

class Polyline
{
 private:

 Vector* sequenza;
 int numero, massimo;

 public:

 Polyline(int _massimo)
 {
  numero = 0;
  massimo = _massimo;
  sequenza = new Vector[massimo];
 }

 ~Polyline(void)
 {
  delete sequenza;
 }

 int getNumeroVettori(void)
 {
  return numero;
 }

 bool addVector(Vector vettore)
 {
  if (numero >= massimo)
    return false;
  if (numero == 0)
    {
     sequenza[numero] = vettore;
     numero++;
	 return true;
    }
  if (sequenza[numero-1].getX1() == vettore.getX0() && sequenza[numero-1].getY1() == vettore.getY0())
    {
     sequenza[numero] = vettore;
     numero++;
	 return true;
    }
  return false;
 }

 Vector getVector(int posizione)
 {
  Vector tmp; // vettore nullo

  if (posizione>=1 && posizione<=numero)
    {
     return sequenza[posizione-1];
    }
  
  return tmp;
 }

 double length(void)
 {
  double lunghezza = 0;

  for (int i=0; i<numero; i++)
     lunghezza += sequenza[i].length();

  return lunghezza;
 }

 bool simple(void)
 {
  for (int i=0; i<numero-1; i++)
     for (int j=i+1; j<numero; j++)
        if (sequenza[i].intersect(sequenza[j]))
          return false;
  return true;
 }
};




int main(void)
{
 Polyline p(10);
 Vector v;

 v = Vector(0,0,1,1);
 p.addVector(v);
 v = Vector(1,1,0,1);
 p.addVector(v);
 v = Vector(0,1,1,0);
 p.addVector(v);

 printf("Numero vettori: %i\r\n", p.getNumeroVettori());
 printf("Lunghezza: %f\r\n",p.length());
 if (p.simple())
   printf("Semplice\r\n");
 else
	 printf("Intrecciata\r\n");
}
