
#include <cstring>
#include <cstdio>

struct RECORD
{
 char tipo;
 char nominativo[32];
 char descrizione[1024];
};

class Pratica
{
 private:

 static int progressivo;
 int numero;
 char tipo;
 char nominativo[32];
 char descrizione[1024];

 public:

 Pratica(void)
 {
  numero = 0;
  tipo = '?';
  strcpy(nominativo, "");
  strcpy(descrizione, "");
 }

 Pratica(char _tipo, char _nominativo[], char _descrizione[])
 {
  progressivo++;
  numero = progressivo;
  tipo = _tipo;
  strcpy(nominativo, _nominativo);
  strcpy(descrizione, _descrizione);
 }

 int getNumero(void) { return numero; }
 char getTipo(void) { return tipo; }
 void getNominativo(char* _nominativo) { strcpy(_nominativo, nominativo); }
 void getDescrizione(char* _descrizione) { strcpy(_descrizione, descrizione); }
};

int Pratica::progressivo = 0;


class Registro
{
 private:
 
 int numero_massimo_pratiche;
 int numero_pratiche;
 int prima_pratica;
 int ultima_pratica;
 Pratica* pratiche;

 public:

 Registro(int _numero_massimo_pratiche)
 {
  FILE* file;
  RECORD record;
  Pratica pratica;
       
  numero_massimo_pratiche = _numero_massimo_pratiche;
  pratiche = new Pratica[numero_massimo_pratiche];
  numero_pratiche = 0;
  prima_pratica = 0;
  ultima_pratica = 0;

  // lettura pratiche da file
  file = fopen("pratiche.bin", "rb");
  if (file != NULL)
    {
     do
	   {
	    if (fread(&record, sizeof(RECORD), 1, file) > 0)
		  {
           pratica = Pratica(record.tipo, record.nominativo, record.descrizione);
		   nuovaPratica(pratica);
		  }
	   }  while (!feof(file));
	 fclose(file);
    }   
 }

~Registro(void)
{
 int posizione, contatore;
 FILE* file;
 RECORD record;
 
 // scrittura pratiche su file
 file = fopen("pratiche.bin", "wb");
 posizione = prima_pratica;
 contatore = 0;
 while (contatore < numero_pratiche)
      {
	   record.tipo = pratiche[posizione].getTipo();
	   pratiche[posizione].getNominativo(record.nominativo);
	   pratiche[posizione].getDescrizione(record.descrizione);
	   fwrite(&record, sizeof(record), 1, file);
	   posizione = (posizione+1) % numero_massimo_pratiche;
	   contatore++;
      }
 fclose(file);
 delete pratiche;
}

int numeroPratiche(void) { return numero_pratiche; }

bool nuovaPratica(Pratica pratica)
{
 if (numero_pratiche >= numero_massimo_pratiche)
   return false;
 pratiche[ultima_pratica] = pratica;
 ultima_pratica = (ultima_pratica+1) % numero_massimo_pratiche;
 numero_pratiche++;
 return true;
}

Pratica evadiPratica(void)
{
 Pratica pratica; // pratica invalida

 if (numero_pratiche == 0)
   return pratica;
 pratica = pratiche[prima_pratica];
 prima_pratica = (prima_pratica+1) % numero_massimo_pratiche;
 numero_pratiche--;
 return pratica;
}

Pratica ricercaPratica(int numero)
{
 Pratica pratica; // pratica invalida
 int posizione, contatore;

 if (numero_pratiche == 0)
   return pratica;
 
 // ricerca pratica
 posizione = prima_pratica;
 contatore = 0;
 while (contatore < numero_pratiche)
      {
	   if (pratiche[posizione].getNumero() == numero)
	     return pratiche[posizione];
	   posizione = (posizione+1) % numero_massimo_pratiche;
	   contatore++;
      }
 return pratica;
}

int ricercaPratiche(char tipo, Pratica* elenco)
{
 int posizione, contatore, numero = 0;

 if (numero_pratiche == 0)
   return 0;

 // ricerca pratiche
 posizione = prima_pratica;
 contatore = 0;
 while (contatore < numero_pratiche)
      {
	   if (pratiche[posizione].getTipo() == tipo)
	     {
		  elenco[numero] = pratiche[posizione];
		  numero++;
	     }
	   posizione = (posizione+1) % numero_massimo_pratiche;
	   contatore++;
      }
 return numero;
}
};




int main(void)
{
 Registro registro(100);
 Pratica pratica;
 Pratica elenco[10];
 int i, n;
 char nominativo[32], descrizione[1024];

 pratica = Pratica('X', "Paolino Paperino", "Richiesta di credito");
 registro.nuovaPratica(pratica);
 pratica = Pratica('Y', "Paperon de Paperoni", "Richiesta esenzione tasse");
 registro.nuovaPratica(pratica);
 pratica = Pratica('Z', "Archimede Pitagorico", "Richiesta brevetto");
 registro.nuovaPratica(pratica);
 pratica = Pratica('X', "Paperon de Paperoni", "Richiesta pensione per arcimiliardari");
 registro.nuovaPratica(pratica);
 pratica = Pratica('X', "Archimede Pitagorico", "Richiesta sovvenzione inventori");
 registro.nuovaPratica(pratica);

 printf("%i pratiche inevase\r\n", registro.numeroPratiche());
 n = registro.ricercaPratiche('X', elenco);
 for (i=0 ; i<n; i++)
    {
	 elenco[i].getNominativo(nominativo);
	 elenco[i].getDescrizione(descrizione);
     printf("%i %c %s %s\r\n", elenco[i].getNumero(), elenco[i].getTipo(), nominativo, descrizione);
    }
 pratica = registro.ricercaPratica(3);
 pratica.getNominativo(nominativo);
 pratica.getDescrizione(descrizione);
 printf("%i %c %s %s\r\n", pratica.getNumero(), pratica.getTipo(), nominativo, descrizione);
 pratica = registro.evadiPratica();
 pratica.getNominativo(nominativo);
 pratica.getDescrizione(descrizione);
 printf("%i %c %s %s\r\n", pratica.getNumero(), pratica.getTipo(), nominativo, descrizione);
 printf("%i pratiche inevase\r\n", registro.numeroPratiche());
 pratica = registro.evadiPratica();
 pratica.getNominativo(nominativo);
 pratica.getDescrizione(descrizione);
 printf("%i %c %s %s\r\n", pratica.getNumero(), pratica.getTipo(), nominativo, descrizione);
 printf("%i pratiche inevase\r\n", registro.numeroPratiche());
}
